"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _traverse = _interopRequireDefault(require("@babel/traverse"));

var types = _interopRequireWildcard(require("@babel/types"));

var _utils = require("./utils");

var astUtil = _interopRequireWildcard(require("./utils/ast"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _default = function _default() {
  return {
    name: 'babel-plugin-jsx-sfc',
    visitor: {
      Program: function Program(_path, state) {
        (0, _traverse["default"])(_path.node, {
          /*
            const App = sfc(
              {
                template({ data }) {
                  ...
                },
            
                Component: (props) => {
                  ...
                },
            
                styles: () => { ... }
              },
              () => ({
                utils: { ... }
              })
            );
              ↓ ↓ ↓ ↓ ↓ ↓
              const $sfcOptions_lineNo = sfc.createOptions(
              {
                template({ data }) {
                  ...
                },
            
                styles: () => ({
                  ...
                })
              },
              () => ({
                utils: { ... }
              })
            );
              const Sfc_lineNo = (props) => {
              ...
              return $sfcOptions_lineNo.template({ ... });
            };
              const App = sfc(Sfc_lineNo, $sfcOptions_lineNo);
          */
          CallExpression: {
            enter: function enter(path) {
              var _state$opts;

              var callee = path.node.callee;
              var importedLib = state === null || state === void 0 ? void 0 : (_state$opts = state.opts) === null || _state$opts === void 0 ? void 0 : _state$opts.importedLib;
              /**
               * 0: nothing
               * 1: const App = sfc()({ ... })
               * 2: const App = sfc({ ... })
               * 3: const App = sfc.forwardRef()({ ... })
               * 4: const App = sfc.forwardRef({ ... })
               */

              var sfcType = 0;

              if (types.isCallExpression(callee) && astUtil.isCalleeImportedBySfc(callee.callee, path, importedLib)) {
                if (types.isMemberExpression(callee.callee) && types.isIdentifier(callee.callee.property) && callee.callee.property.name === _utils.SFC_FORWARD_REF) {
                  sfcType = 3;
                } else {
                  sfcType = 1;
                }
              } else if (astUtil.isCalleeImportedBySfc(callee, path, importedLib)) {
                if (types.isMemberExpression(callee) && types.isIdentifier(callee.property) && callee.property.name === _utils.SFC_FORWARD_REF) {
                  sfcType = 4;
                } else {
                  sfcType = 2;
                }
              }

              if (sfcType) {
                var sfcArguments = path.node.arguments;

                if (types.isObjectExpression(sfcArguments === null || sfcArguments === void 0 ? void 0 : sfcArguments[0])) {
                  var _path$node, _path$node$loc;

                  var componentProp = sfcArguments[0].properties.find(function (prop) {
                    return types.isObjectProperty(prop) && types.isIdentifier(prop.key) && prop.key.name === _utils.SFC_COMPONENT;
                  });
                  var componentMethod = sfcArguments[0].properties.find(function (prop) {
                    return types.isObjectMethod(prop) && types.isIdentifier(prop.key) && prop.key.name === _utils.SFC_COMPONENT;
                  });

                  if (!componentProp && !componentMethod) {
                    return;
                  }

                  var indexInProps = sfcArguments[0].properties.indexOf(componentProp ? componentProp : componentMethod);
                  var lineNo = path === null || path === void 0 ? void 0 : (_path$node = path.node) === null || _path$node === void 0 ? void 0 : (_path$node$loc = _path$node.loc) === null || _path$node$loc === void 0 ? void 0 : _path$node$loc.start.line;
                  var sfcOptionsName = (0, _utils.getOptionsName)(lineNo); // todo: has template?

                  /*
                    Component: props => {
                      return { firstName: 'joe' };
                    }
                      ↓ ↓ ↓ ↓ ↓ ↓ 
                      Component: props => {
                      return $sfcOptions_lineNo.template({ firstName: 'joe' });
                    }
                  */

                  var componentFuncPath;

                  if (componentProp) {
                    componentFuncPath = path.get("arguments.0.properties.".concat(indexInProps, ".value"));
                  } else {
                    componentFuncPath = path.get("arguments.0.properties.".concat(indexInProps));
                  }

                  var funcBlockPath = componentFuncPath.get('body');
                  var returnArgPath = componentFuncPath.get('body.body').find(function (p) {
                    return p.isReturnStatement();
                  });
                  var firstPropParamPath = componentFuncPath.get('params.0');

                  if (firstPropParamPath) {
                    if (types.isObjectPattern(firstPropParamPath.node)) {
                      // const { styles } = { ...props, ...$sfcOptions_lineNo };
                      var propsName = 'props';
                      funcBlockPath.unshiftContainer('body', types.variableDeclaration('const', [types.variableDeclarator(firstPropParamPath.node, types.objectExpression([types.spreadElement(types.identifier(propsName)), types.spreadElement(types.identifier(sfcOptionsName))]))]));
                      firstPropParamPath.replaceWith(types.identifier(propsName));
                    } else {
                      // props = { ...props, ...$sfcOptions_lineNo };
                      var _propsName = firstPropParamPath.node.name;
                      funcBlockPath.unshiftContainer('body', types.expressionStatement(types.assignmentExpression('=', types.identifier(_propsName), types.objectExpression([types.spreadElement(types.identifier(_propsName)), types.spreadElement(types.identifier(sfcOptionsName))]))));
                    }
                  } // return $sfcOptions_lineNo.template({ ... });


                  if (types.isObjectExpression(returnArgPath.node.argument)) {
                    returnArgPath.replaceWith(types.returnStatement(types.callExpression(types.memberExpression(types.identifier(sfcOptionsName), types.identifier('template')), [returnArgPath.node.argument])));
                  }

                  sfcArguments[0].properties.splice(indexInProps, 1); // const $sfcOptions_lineNo = ...

                  var componentVariable = path.findParent(function (path) {
                    return path.isVariableDeclaration() || path.isExportDefaultDeclaration();
                  });
                  var sfcOptionsPath = componentVariable === null || componentVariable === void 0 ? void 0 : componentVariable.insertBefore(types.variableDeclaration('const', [types.variableDeclarator(types.identifier(sfcOptionsName), types.callExpression(types.memberExpression(types.identifier(_utils.SFC_FUNC), types.identifier(_utils.SFC_CREATE_OPTIONS)), sfcArguments))]));
                  var actualComponentFunc;

                  if (componentProp) {
                    actualComponentFunc = componentProp.value;
                  } else {
                    actualComponentFunc = types.arrowFunctionExpression(componentMethod.params, componentMethod.body);
                  }

                  if (sfcType <= 2) {
                    var sfcName = (0, _utils.getSfcName)(lineNo);
                    componentVariable === null || componentVariable === void 0 ? void 0 : componentVariable.insertBefore(types.variableDeclaration('const', [types.variableDeclarator(types.identifier(sfcName), actualComponentFunc)]));

                    if (types.isVariableDeclaration(componentVariable === null || componentVariable === void 0 ? void 0 : componentVariable.node)) {
                      var _componentVariable$no, _componentVariable$no2;

                      var declarationId = componentVariable === null || componentVariable === void 0 ? void 0 : (_componentVariable$no = componentVariable.node.declarations) === null || _componentVariable$no === void 0 ? void 0 : (_componentVariable$no2 = _componentVariable$no[0]) === null || _componentVariable$no2 === void 0 ? void 0 : _componentVariable$no2.id;

                      if (types.isIdentifier(declarationId)) {
                        var componentName = declarationId.name;
                        componentVariable === null || componentVariable === void 0 ? void 0 : componentVariable.insertBefore(types.expressionStatement(types.assignmentExpression('=', types.memberExpression(types.identifier(sfcName), types.identifier('displayName')), types.stringLiteral(componentName))));
                      }
                    }

                    path.replaceWith(types.callExpression(types.identifier(_utils.SFC_FUNC), [types.identifier(sfcName), types.identifier(sfcOptionsName)]));
                  } else {
                    path.replaceWith(types.callExpression(types.memberExpression(types.identifier(_utils.SFC_FUNC), types.identifier(_utils.SFC_FORWARD_REF)), [actualComponentFunc, types.identifier(sfcOptionsName)]));
                  } // console.log(generate(path.node).code);
                  // console.log(generate(sfcOptionsPath[0].node).code);

                }
              }
            }
          }
        });
      }
    }
  };
};

exports["default"] = _default;